{- |

mtlstats
Copyright (C) Rhéal Lamothe
<rheal.lamothe@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

-}

module Mtlstats (app) where

import Brick.AttrMap (AttrMap, forceAttrMap)
import Brick.Main (App (..), halt, showFirstCursor)
import Brick.Types (BrickEvent (VtyEvent), Widget)
import Brick.Util (on)
import Brick.Widgets.Core (fill)
import Control.Monad.State.Class (gets)
import Graphics.Vty.Attributes.Color (blue, white)
import Graphics.Vty.Input.Events
  ( Event (EvKey)
  , Modifier (MCtrl)
  , Key (KChar)
  )

import Mtlstats.Control
import Mtlstats.Types

-- | The main application
app :: App ProgState () ()
app = App
  { appDraw         = draw
  , appChooseCursor = showFirstCursor
  , appHandleEvent  = handler
  , appStartEvent   = return ()
  , appAttrMap      = const myAttrMap
  }

draw :: ProgState -> [Widget ()]
draw s =
  [ drawController (dispatch s) s
  , fill ' '
  ]

handler :: Handler ()
handler (VtyEvent (EvKey (KChar 'c') [MCtrl])) = halt
handler e = do
  c <- gets dispatch
  handleController c e

myAttrMap :: AttrMap
myAttrMap = forceAttrMap (white `on` blue)
